@@ -25,7 +25,7 @@
 #include "capplet-util.h"
 
 /* This must match the number of images on the 2nd page in the UI file */
-#define MAX_ENROLL_STAGES 3
+#define MAX_ENROLL_STAGES 5
 
 static DBusGProxy *manager = NULL;
 static DBusGConnection *connection = NULL;
@@ -220,7 +220,7 @@ delete_fingerprints_question (GtkBuilder *dialog, GtkWidget *enable, GtkWidget *
 
 	button = gtk_button_new_with_mnemonic (_("_Delete Fingerprints"));
 	gtk_button_set_image (GTK_BUTTON (button), gtk_image_new_from_stock (GTK_STOCK_DELETE, GTK_ICON_SIZE_BUTTON));
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default (button, TRUE);
 	gtk_widget_show (button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (question), button, GTK_RESPONSE_OK);
 
@@ -301,13 +301,27 @@ selected_finger (GtkBuilder *dialog)
 static void
 finger_radio_button_toggled (GtkToggleButton *button, EnrollData *data)
 {
+	GtkBuilder *dialog = data->dialog;
+	char *msg;
+
 	data->finger = selected_finger (data->dialog);
+
+	msg = g_strdup_printf (finger_str_to_msg (data->finger, data->is_swipe), data->name);
+	gtk_label_set_text (GTK_LABEL (WID("enroll-label")), msg);
+	g_free (msg);
 }
 
 static void
 finger_combobox_changed (GtkComboBox *combobox, EnrollData *data)
 {
+	GtkBuilder *dialog = data->dialog;
+	char *msg;
+
 	data->finger = selected_finger (data->dialog);
+
+	msg = g_strdup_printf (finger_str_to_msg (data->finger, data->is_swipe), data->name);
+	gtk_label_set_text (GTK_LABEL (WID("enroll-label")), msg);
+	g_free (msg);
 }
 
 static void
