@@ -6,6 +6,7 @@
 #include <sys/stat.h>
 #include <dirent.h>
 #include <glib/gi18n.h>
+#include <gmodule.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gio/gio.h>
@@ -404,10 +405,13 @@ read_icon_theme (GFile *icon_theme_uri)
   if (icon_theme_ditem == NULL)
     return NULL;
 
-  name = gnome_desktop_item_get_string (icon_theme_ditem, "Icon Theme/Name");
-  if (name == NULL) {
-    gnome_desktop_item_unref (icon_theme_ditem);
-    return NULL;
+  name = gnome_desktop_item_get_localestring (icon_theme_ditem, "Icon Theme/Name");
+  if (!name) {
+    name = gnome_desktop_item_get_localestring (icon_theme_ditem, GNOME_DESKTOP_ITEM_NAME);
+    if (!name) {
+      gnome_desktop_item_unref (icon_theme_ditem);
+      return NULL;
+    }
   }
 
   /* If index.theme has no Directories entry, it is only a cursor theme */
@@ -685,7 +689,8 @@ read_cursor_fonts (void)
 
 static void
 handle_change_signal (gpointer             data,
-                      GnomeThemeChangeType change_type)
+                      GnomeThemeChangeType change_type,
+                      GnomeThemeElement    element_type)
 {
 #ifdef DEBUG
   gchar *type_str = NULL;
@@ -700,7 +705,7 @@ handle_change_signal (gpointer             data,
 
   for (list = callbacks; list; list = list->next) {
     ThemeCallbackData *callback_data = list->data;
-    (* callback_data->func) (theme, change_type, callback_data->data);
+    (* callback_data->func) (theme, change_type, element_type, callback_data->data);
   }
 
 #ifdef DEBUG
@@ -711,12 +716,11 @@ handle_change_signal (gpointer             data,
   else if (theme->type == GNOME_THEME_TYPE_CURSOR)
     type_str = "cursor";
   else if (theme->type == GNOME_THEME_TYPE_REGULAR) {
-    GnomeThemeInfo *rtheme = (GnomeThemeInfo *) theme;
-    if (rtheme->has_gtk)
+    if (element_type & GNOME_THEME_GTK_2)
       element_str = "gtk-2";
-    else if (rtheme->has_keybinding)
+    else if (element_type & GNOME_THEME_GTK_2_KEYBINDING)
       element_str = "keybinding";
-    else if (rtheme->has_metacity)
+    else if (element_type & GNOME_THEME_METACITY)
       element_str = "metacity";
   }
 
@@ -781,7 +785,7 @@ update_theme_index (GFile            *index_uri,
 
       g_hash_table_insert (theme_hash_by_uri, g_strdup (common_theme_dir), theme_info);
       add_theme_to_hash_by_name (theme_hash_by_name, theme_info);
-      handle_change_signal (theme_info, GNOME_THEME_CHANGE_CREATED);
+      handle_change_signal (theme_info, GNOME_THEME_CHANGE_CREATED, key_element);
     }
   } else {
     gboolean theme_used_to_exist = FALSE;
@@ -803,11 +807,11 @@ update_theme_index (GFile            *index_uri,
     }
 
     if (theme_exists && theme_used_to_exist) {
-      handle_change_signal (theme_info, GNOME_THEME_CHANGE_CHANGED);
+      handle_change_signal (theme_info, GNOME_THEME_CHANGE_CHANGED, key_element);
     } else if (theme_exists && !theme_used_to_exist) {
-      handle_change_signal (theme_info, GNOME_THEME_CHANGE_CREATED);
+      handle_change_signal (theme_info, GNOME_THEME_CHANGE_CREATED, key_element);
     } else if (!theme_exists && theme_used_to_exist) {
-      handle_change_signal (theme_info, GNOME_THEME_CHANGE_DELETED);
+      handle_change_signal (theme_info, GNOME_THEME_CHANGE_DELETED, key_element);
     }
 
     if (!theme_info->has_metacity && !theme_info->has_keybinding && !theme_info->has_gtk) {
@@ -903,7 +907,7 @@ update_common_theme_dir_index (GFile         *theme_index_uri,
     if (theme_exists) {
       g_hash_table_insert (hash_by_uri, g_strdup (common_theme_dir), theme_info);
       add_theme_to_hash_by_name (hash_by_name, theme_info);
-      handle_change_signal (theme_info, GNOME_THEME_CHANGE_CREATED);
+      handle_change_signal (theme_info, GNOME_THEME_CHANGE_CREATED, 0);
     }
   } else {
     if (theme_exists) {
@@ -913,7 +917,7 @@ update_common_theme_dir_index (GFile         *theme_index_uri,
         remove_theme_from_hash_by_name (hash_by_name, old_theme_info);
         g_hash_table_insert (hash_by_uri, g_strdup (common_theme_dir), theme_info);
         add_theme_to_hash_by_name (hash_by_name, theme_info);
-        handle_change_signal (theme_info, GNOME_THEME_CHANGE_CHANGED);
+        handle_change_signal (theme_info, GNOME_THEME_CHANGE_CHANGED, 0);
         theme_free (old_theme_info);
       } else {
         theme_free (theme_info);
@@ -922,7 +926,7 @@ update_common_theme_dir_index (GFile         *theme_index_uri,
       g_hash_table_remove (hash_by_uri, common_theme_dir);
       remove_theme_from_hash_by_name (hash_by_name, old_theme_info);
 
-      handle_change_signal (old_theme_info, GNOME_THEME_CHANGE_DELETED);
+      handle_change_signal (old_theme_info, GNOME_THEME_CHANGE_DELETED, 0);
       theme_free (old_theme_info);
     }
   }
@@ -1437,10 +1441,8 @@ gtk_theme_info_missing_engine (const gchar *gtk_theme, gboolean nameOnly)
     g_free (gtkrc);
 
     for (l = engines; l; l = l->next) {
-      gchar *lib = g_strconcat ("lib", l->data, ".so", NULL);
-      gchar *full = g_build_filename (GTK_ENGINE_DIR, lib, NULL);
+      gchar *full = g_module_build_path (GTK_ENGINE_DIR, l->data);
 
-      g_free (lib);
       found = g_file_test (full, G_FILE_TEST_EXISTS);
 
       if (!found) {
@@ -1820,7 +1822,9 @@ gnome_theme_init ()
   GFile *top_theme_dir;
   gchar *top_theme_dir_string;
   static gboolean initted = FALSE;
-  const gchar *gtk_data_dir;
+  gchar **search_path;
+  gint i, n;
+
   if (initted)
     return;
 
@@ -1852,44 +1856,33 @@ gnome_theme_init ()
   add_top_theme_dir_monitor (top_theme_dir, 0, NULL);
   g_object_unref (top_theme_dir);
 
-  /* The weird /usr/share/icons */
-  top_theme_dir = g_file_new_for_path ("/usr/share/icons");
-  add_top_icon_theme_dir_monitor (top_theme_dir, 2, NULL);
+  /* ~/.icons */
+  top_theme_dir_string = g_build_filename (g_get_home_dir (), ".icons", NULL);
+  top_theme_dir = g_file_new_for_path (top_theme_dir_string);
+  g_free (top_theme_dir_string);
+  if (!g_file_query_exists (top_theme_dir, NULL))
+    g_file_make_directory (top_theme_dir, NULL, NULL);
   g_object_unref (top_theme_dir);
 
-  /* $datadir/icons */
-  gtk_data_dir = g_getenv ("GTK_DATA_PREFIX");
-  if (gtk_data_dir)
-    top_theme_dir_string = g_build_filename (gtk_data_dir, "share", "icons", NULL);
-  else
-    top_theme_dir_string = g_build_filename (INSTALL_PREFIX, "share", "icons", NULL);
+  /* icon theme search path */
+  gtk_icon_theme_get_search_path (gtk_icon_theme_get_default (), &search_path, &n);
+  for (i = 0; i < n; ++i) {
+    top_theme_dir = g_file_new_for_path (search_path[i]);
+    add_top_icon_theme_dir_monitor (top_theme_dir, i, NULL);
+    g_object_unref (top_theme_dir);
+  }
+  g_strfreev (search_path);
 
 #ifdef XCURSOR_ICONDIR
   /* if there's a separate xcursors dir, add that as well */
   if (strcmp (XCURSOR_ICONDIR, top_theme_dir_string) &&
       strcmp (XCURSOR_ICONDIR, "/usr/share/icons")) {
     top_theme_dir = g_file_new_for_path (XCURSOR_ICONDIR);
-    if (g_file_query_exists (top_theme_dir, NULL))
-      add_top_icon_theme_dir_monitor (top_theme_dir, 1, NULL);
+    add_top_icon_theme_dir_monitor (top_theme_dir, 1, NULL);
     g_object_unref (top_theme_dir);
   }
 #endif
 
-  top_theme_dir = g_file_new_for_path (top_theme_dir_string);
-  g_free (top_theme_dir_string);
-  add_top_icon_theme_dir_monitor (top_theme_dir, 1, NULL);
-  g_object_unref (top_theme_dir);
-
-  /* ~/.icons */
-  top_theme_dir_string = g_build_filename (g_get_home_dir (), ".icons", NULL);
-  top_theme_dir = g_file_new_for_path (top_theme_dir_string);
-  g_free (top_theme_dir_string);
-
-  if (!g_file_query_exists (top_theme_dir, NULL))
-    g_file_make_directory (top_theme_dir, NULL, NULL);
-  add_top_icon_theme_dir_monitor (top_theme_dir, 0, NULL);
-  g_object_unref (top_theme_dir);
-
 #ifdef HAVE_XCURSOR
   /* make sure we have the default theme */
   if (!gnome_theme_cursor_info_find ("default"))
