@@ -29,13 +29,12 @@
 #include <glib/gi18n.h>
 #include <string.h>
 #include <gconf/gconf-client.h>
-#include <glade/glade.h>
 
 #include "capplet-util.h"
 
 #include "gnome-keyboard-properties-xkb.h"
 
-static GladeXML *chooser_dialog = NULL;
+static GtkBuilder *chooser_dialog = NULL;
 static const char *current1st_level_id = NULL;
 static GSList *option_checks_list = NULL;
 static GtkWidget *current_none_radio = NULL;
@@ -184,12 +183,13 @@ option_focused_cb (GtkWidget * widget, GdkEventFocus * event,
 		   gpointer data)
 {
 	GtkScrolledWindow *win = GTK_SCROLLED_WINDOW (data);
-	GtkAllocation *alloc = &widget->allocation;
+	GtkAllocation alloc;
 	GtkAdjustment *adj;
 
+	gtk_widget_get_allocation (widget, &alloc);
 	adj = gtk_scrolled_window_get_vadjustment (win);
-	gtk_adjustment_clamp_page (adj, alloc->y,
-				   alloc->y + alloc->height);
+	gtk_adjustment_clamp_page (adj, alloc.y,
+				   alloc.y + alloc.height);
 
 	return FALSE;
 }
@@ -211,7 +211,7 @@ option_toggled_cb (GtkWidget * checkbutton, gpointer data)
    the top of this file. */
 static void
 xkb_options_add_option (XklConfigRegistry * config_registry,
-			XklConfigItem * config_item, GladeXML * dialog)
+			XklConfigItem * config_item, GtkBuilder * dialog)
 {
 	GtkWidget *option_check;
 	gchar *utf_option_name = xci_desc_to_utf8 (config_item);
@@ -302,7 +302,7 @@ xkb_option_checks_compare (GtkWidget * chk1, GtkWidget * chk2)
    add widgets for all the options in the group. */
 static void
 xkb_options_add_group (XklConfigRegistry * config_registry,
-		       XklConfigItem * config_item, GladeXML * dialog)
+		       XklConfigItem * config_item, GtkBuilder * dialog)
 {
 	GtkWidget *align, *vbox, *option_check;
 	gboolean allow_multiple_selection =
@@ -348,7 +348,7 @@ xkb_options_add_group (XklConfigRegistry * config_registry,
 			  (GCompareFunc) xkb_option_checks_compare);
 	while (option_checks_list) {
 		option_check = GTK_WIDGET (option_checks_list->data);
-		gtk_box_pack_start_defaults (GTK_BOX (vbox), option_check);
+		gtk_box_pack_start (GTK_BOX (vbox), option_check, TRUE, TRUE, 0);
 		option_checks_list = option_checks_list->next;
 	}
 	/* free it */
@@ -379,7 +379,7 @@ xkb_options_expanders_compare (GtkWidget * expander1,
 
 /* Create widgets to represent the options made available by the backend */
 void
-xkb_options_load_options (GladeXML * dialog)
+xkb_options_load_options (GtkBuilder * dialog)
 {
 	GtkWidget *opts_vbox = WID ("options_vbox");
 	GSList *expanders_list;
@@ -416,29 +416,37 @@ xkb_options_load_options (GladeXML * dialog)
 static void
 chooser_response_cb (GtkDialog * dialog, gint response, gpointer data)
 {
-	if (response == GTK_RESPONSE_CLOSE) {
-		/* just cleanup */
-		GSList *expanders_list =
-		    g_object_get_data (G_OBJECT (dialog), EXPANDERS_PROP);
-		g_object_set_data (G_OBJECT (dialog), EXPANDERS_PROP,
-				   NULL);
-		g_slist_free (expanders_list);
+	switch (response) {
+	case GTK_RESPONSE_HELP:
+		capplet_help (GTK_WINDOW (dialog),
+			      "prefs-keyboard-layoutoptions");
+		break;
+	case GTK_RESPONSE_CLOSE:{
+			/* just cleanup */
+			GSList *expanders_list =
+			    g_object_get_data (G_OBJECT (dialog),
+					       EXPANDERS_PROP);
+			g_object_set_data (G_OBJECT (dialog),
+					   EXPANDERS_PROP, NULL);
+			g_slist_free (expanders_list);
 
-		gtk_widget_destroy (GTK_WIDGET (dialog));
-		chooser_dialog = NULL;
+			gtk_widget_destroy (GTK_WIDGET (dialog));
+			chooser_dialog = NULL;
+		}
+		break;
 	}
 }
 
 /* Create popup dialog */
 void
-xkb_options_popup_dialog (GladeXML * dialog)
+xkb_options_popup_dialog (GtkBuilder * dialog)
 {
 	GtkWidget *chooser;
 
-	chooser_dialog =
-	    glade_xml_new (GNOMECC_GLADE_DIR
-			   "/gnome-keyboard-properties.glade",
-			   "xkb_options_dialog", NULL);
+	chooser_dialog = gtk_builder_new ();
+    gtk_builder_add_from_file (chooser_dialog, GNOMECC_UI_DIR
+                               "/gnome-keyboard-properties-options-dialog.ui",
+                               NULL);
 
 	chooser = CWID ("xkb_options_dialog");
 	gtk_window_set_transient_for (GTK_WINDOW (chooser),
@@ -468,7 +476,7 @@ xkb_options_update_option_counters (XklConfigRegistry * config_registry,
 /* Respond to a change in the xkb gconf settings */
 static void
 xkb_options_update (GConfClient * client,
-		    guint cnxn_id, GConfEntry * entry, GladeXML * dialog)
+		    guint cnxn_id, GConfEntry * entry, GtkBuilder * dialog)
 {
 	/* Updating options is handled by gconf notifies for each widget
 	   This is here to avoid calling it N_OPTIONS times for each gconf
@@ -499,7 +507,7 @@ xkb_options_update (GConfClient * client,
 }
 
 void
-xkb_options_register_gconf_listener (GladeXML * dialog)
+xkb_options_register_gconf_listener (GtkBuilder * dialog)
 {
 	gconf_client_notify_add (xkb_gconf_client,
 				 GKBD_KEYBOARD_CONFIG_KEY_OPTIONS,
